@@ -1,18 +1,30 @@
+import json
 import os
 import re
 import sys
 import time
 from datetime import datetime, timedelta
+from distutils.version import LooseVersion
 from io import BytesIO
 from multiprocessing import cpu_count
 from multiprocessing.pool import ThreadPool
 
-import av                                   # av
-import click                                # click
-from lxml import etree                      # lxml
-from lxml.etree import QName, SubElement    # lxml
-from requests import get                    # requests
-from tqdm import tqdm                       # tqdm
+import av                                               # av
+import click                                            # click
+import pkg_resources                                    # setuptools
+from lxml import etree                                  # lxml
+from lxml.etree import QName, SubElement                # lxml
+import requests                                         # requests
+from requests.adapters import HTTPAdapter               # requests
+from requests.packages.urllib3.util.retry import Retry  # requests
+from tqdm import tqdm                                   # tqdm
+
+s = requests.Session()
+retry = Retry(connect=5, backoff_factor=0.5)
+adapter = HTTPAdapter(max_retries=retry)
+s.mount('http://', adapter)
+s.mount('https://', adapter)
+get = s.get
 
 
 class Stream:
@@ -44,9 +56,12 @@ def local_to_utc(dt):
 
 def get_mpd_data(video_url):
     page = get(video_url).text
-    mpd_link = (
-        page.split('dashManifestUrl\\":\\"')[-1].split('\\"')[0].replace("\/", "/")
-    )
+    if 'dashManifestUrl\\":\\"' in page:
+        mpd_link = page.split('dashManifestUrl\\":\\"')[-1].split('\\"')[0].replace("\/", "/")
+    elif 'dashManifestUrl":"' in page:
+        mpd_link = page.split('dashManifestUrl":"')[-1].split('"')[0].replace("\/", "/")
+    else:
+        return None
     return get(mpd_link).text
 
 
@@ -61,6 +76,9 @@ def process_mpd(mpd_data):
         + len(tree.findall(".//def:S", nsmap))
         - 1
     )
+    # Float stupidity for now cause Python doesnt know how to parse this
+    # TODO: Make segments actually work without these workarounds
+    seg_len = int(float(root.attrib["minimumUpdatePeriod"][2:-1]))
     attribute_sets = tree.findall(".//def:Period/def:AdaptationSet", nsmap)
     v_streams = []
     a_streams = []
@@ -78,7 +96,7 @@ def process_mpd(mpd_data):
                 v_streams.append(Stream(stream_type, bitrate, codec, quality, base_url))
     a_streams.sort(key=lambda x: x.bitrate, reverse=True)
     v_streams.sort(key=lambda x: x.bitrate, reverse=True)
-    return a_streams, v_streams, total_seg, d_time
+    return a_streams, v_streams, total_seg, d_time, seg_len
 
 
 def info(a, v, m, s):
@@ -168,7 +186,7 @@ def mux_to_file(output, aud, vid):
 def check_if_exists(output):
     if os.path.exists(output):
         yn = input(f"File '{output}' already exists. Overwrite? [y/N] ").lower()
-        if len(yn) > 0 and yn[0] == "y":
+        if yn and yn[0] == "y":
             os.remove(output)
             return True
         else:
@@ -185,7 +203,7 @@ def parse_datetime(inp, utc=True):
             today = datetime.today()
             if not ('d' in fmt):
                 d_time = d_time.replace(year=today.year, month=today.month,day=today.day)
-            if not ('y' in fmt):
+            if not ('Y' in fmt):
                 d_time = d_time.replace(year=today.year)
             if utc:
                 return d_time
@@ -197,7 +215,7 @@ def parse_datetime(inp, utc=True):
 
 def parse_duration(inp):
     x = re.findall("([0-9]+[hmsHMS])", inp)
-    if len(x) == 0:
+    if not x:
         try:
             number = int(inp)
         except:
@@ -214,6 +232,21 @@ def parse_duration(inp):
                 total_seconds += int(chunk[:-1])
         return total_seconds
 
+def check_for_update():
+    # Ugly code... if you have a better idea please help
+    try:
+        local_version = LooseVersion(pkg_resources.get_distribution("youtube_dash_dl").version)
+    except:
+        return
+    try:
+        req = get("https://pypi.org/pypi/youtube-dash-dl/json")
+        online_version = LooseVersion(json.loads(req.text)['info']['version'])
+    except:
+        return
+
+    if online_version > local_version:
+        print(f"Update avaliable!\nYou should probably update with 'pip install --upgrade youtube_dash_dl'\nLocal version: {local_version} | Online version: {online_version}\n")
+
 
 @click.command()
 @click.argument("url")
@@ -227,14 +260,19 @@ def parse_duration(inp):
 @click.option("-d", "--duration", help="Duration of the download.")
 @click.option("-o", "--output", help="Output file path.")
 def main(**kwargs):
+    check_for_update()
+
     mpd_data = get_mpd_data(kwargs["url"])
-    a, v, m, s = process_mpd(mpd_data)
+    if mpd_data is None:
+        print("Error: Couldn't get MPD data!")
+        return 0
+    a, v, m, s, l = process_mpd(mpd_data)
 
     if kwargs["list_formats"]:
         info(a, v, m, s)
         return 0
 
-    if kwargs["output"] == None:
+    if kwargs["output"] is None:
         print("Error: Missing option '-o' / '--output'!")
         return 0
 
@@ -243,8 +281,8 @@ def main(**kwargs):
         return 0
 
     start_time = (
-        s - timedelta(seconds=m * 2)
-        if kwargs["start"] == None
+        s - timedelta(seconds=m * l)
+        if kwargs["start"] is None
         else parse_datetime(kwargs["start"], kwargs["utc"])
     )
 
@@ -252,29 +290,30 @@ def main(**kwargs):
         print("Error: Couldn't parse start date!")
         return 0
 
-    if kwargs["duration"] == None and kwargs["end"] == None:
-        duration = m * 2
+    if kwargs["duration"] is None and kwargs["end"] is None:
+        duration = m * l
     else:
-        duration = (
-            parse_datetime(kwargs["end"], kwargs["utc"])
-            if kwargs["duration"] == None
-            else parse_duration(kwargs["duration"])
-        )
+        if kwargs["duration"] is None:
+            e_dtime = parse_datetime(kwargs["end"], kwargs["utc"])
+            s_dtime = s if kwargs["start"] is None else parse_datetime(kwargs["start"], kwargs["utc"])
+            duration = (e_dtime - s_dtime).total_seconds()
+        else:
+            duration =  parse_duration(kwargs["duration"])
 
     if duration == -1:
         print("Error: Couldn't parse duration or end date!")
         return 0
 
-    start_segment = m - round((s - start_time).total_seconds() / 2)
+    start_segment = m - round((s - start_time).total_seconds() / l)
     if start_segment < 0:
         start_segment = 0
 
-    end_segment = start_segment + round(duration / 2)
+    end_segment = start_segment + round(duration / l)
     if end_segment > m:
         print("Error: You are requesting segments that dont exist yet!")
         return 0
 
-    download_threads = cpu_count() if kwargs['download_threads'] == None else kwargs['download_threads']
+    download_threads = cpu_count() if kwargs['download_threads'] is None else kwargs['download_threads']
     if download_threads > 4:
         # This is here until we figure out how to use youtube cookies as to not get blocked
         download_threads = 4
